@@ -7,13 +7,16 @@
 package bsonutil
 
 import (
-	"github.com/mongodb/mongo-tools/common/json"
-	. "github.com/smartystreets/goconvey/convey"
-	"gopkg.in/mgo.v2/bson"
 	"testing"
+
+	"github.com/mongodb/mongo-tools/common/json"
+	"github.com/mongodb/mongo-tools/common/testtype"
+	. "github.com/smartystreets/goconvey/convey"
+	"go.mongodb.org/mongo-driver/bson/primitive"
 )
 
 func TestUndefinedValue(t *testing.T) {
+	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
 
 	Convey("When converting JSON with undefined values", t, func() {
 
@@ -23,9 +26,9 @@ func TestUndefinedValue(t *testing.T) {
 				key: json.Undefined{},
 			}
 
-			err := ConvertJSONDocumentToBSON(jsonMap)
+			err := ConvertLegacyExtJSONDocumentToBSON(jsonMap)
 			So(err, ShouldBeNil)
-			So(jsonMap[key], ShouldResemble, bson.Undefined)
+			So(jsonMap[key], ShouldResemble, primitive.Undefined{})
 		})
 
 		Convey(`works for undefined document ('{ "$undefined": true }')`, func() {
@@ -36,9 +39,9 @@ func TestUndefinedValue(t *testing.T) {
 				},
 			}
 
-			err := ConvertJSONDocumentToBSON(jsonMap)
+			err := ConvertLegacyExtJSONDocumentToBSON(jsonMap)
 			So(err, ShouldBeNil)
-			So(jsonMap[key], ShouldResemble, bson.Undefined)
+			So(jsonMap[key], ShouldResemble, primitive.Undefined{})
 		})
 	})
 }
