@@ -11,9 +11,13 @@ package json
 
 import (
 	"testing"
+
+	"github.com/mongodb/mongo-tools/common/testtype"
 )
 
 func TestTagParsing(t *testing.T) {
+	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+
 	name, opts := parseTag("field,foobar,foo")
 	if name != "field" {
 		t.Fatalf("name = %q, want field", name)
