@@ -11,11 +11,13 @@ import (
 	"fmt"
 	"io/ioutil"
 	"os"
+	"regexp"
 	"strings"
 
 	"github.com/mongodb/mongo-tools/common/log"
 	"github.com/mongodb/mongo-tools/common/testtype"
 	. "github.com/smartystreets/goconvey/convey"
+	"github.com/stretchr/testify/require"
 	"go.mongodb.org/mongo-driver/x/mongo/driver/connstring"
 
 	"runtime"
@@ -78,61 +80,69 @@ func TestVerbosityFlag(t *testing.T) {
 		So(optPtr.parser, ShouldNotBeNil)
 
 		Convey("no verbosity flags, Level should be 0", func() {
-			_, err := optPtr.parser.ParseArgs([]string{})
+			_, err := optPtr.CallArgParser([]string{})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 0)
 		})
 
 		Convey("one short verbosity flag, Level should be 1", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"-v"})
+			_, err := optPtr.CallArgParser([]string{"-v"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 1)
 		})
 
 		Convey("three short verbosity flags (consecutive), Level should be 3", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"-vvv"})
+			_, err := optPtr.CallArgParser([]string{"-vvv"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 3)
 		})
 
 		Convey("three short verbosity flags (dispersed), Level should be 3", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"-v", "-v", "-v"})
+			_, err := optPtr.CallArgParser([]string{"-v", "-v", "-v"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 3)
 		})
 
 		Convey("short verbosity flag assigned to 3, Level should be 3", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"-v=3"})
+			_, err := optPtr.CallArgParser([]string{"-v=3"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 3)
 		})
 
 		Convey("consecutive short flags with assignment, only assignment holds", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"-vv=3"})
+			_, err := optPtr.CallArgParser([]string{"-vv=3"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 3)
 		})
 
 		Convey("one long verbose flag, Level should be 1", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"--verbose"})
+			_, err := optPtr.CallArgParser([]string{"--verbose"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 1)
 		})
 
 		Convey("three long verbosity flags, Level should be 3", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"--verbose", "--verbose", "--verbose"})
+			_, err := optPtr.CallArgParser([]string{"--verbose", "--verbose", "--verbose"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 3)
 		})
 
 		Convey("long verbosity flag assigned to 3, Level should be 3", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"--verbose=3"})
+			_, err := optPtr.CallArgParser([]string{"--verbose=3"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 3)
 		})
 
 		Convey("mixed assignment and bare flag, total is sum", func() {
-			_, err := optPtr.parser.ParseArgs([]string{"--verbose", "--verbose=3"})
+			_, err := optPtr.CallArgParser([]string{"--verbose", "--verbose=3"})
+			So(err, ShouldBeNil)
+			So(optPtr.Level(), ShouldEqual, 4)
+		})
+
+		Convey("run CallArgParser multiple times, level should be correct", func() {
+			_, err := optPtr.CallArgParser([]string{"--verbose", "--verbose=3"})
+			So(err, ShouldBeNil)
+			_, err = optPtr.CallArgParser([]string{"--verbose", "--verbose=3"})
 			So(err, ShouldBeNil)
 			So(optPtr.Level(), ShouldEqual, 4)
 		})
@@ -140,19 +150,21 @@ func TestVerbosityFlag(t *testing.T) {
 }
 
 type uriTester struct {
-	Name                 string
-	CS                   connstring.ConnString
-	OptsIn               *ToolOptions
-	OptsExpected         *ToolOptions
-	WithSSL              bool
-	WithGSSAPI           bool
-	ShouldError          bool
-	ShouldAskForPassword bool
+	Name                     string
+	CS                       connstring.ConnString
+	OptsIn                   *ToolOptions
+	OptsExpected             *ToolOptions
+	WithSSL                  bool
+	WithGSSAPI               bool
+	ShouldError              bool
+	AuthShouldAskForPassword bool
 }
 
 func TestParseAndSetOptions(t *testing.T) {
 	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
 
+	FalseValue := false
+
 	Convey("With a matrix of URIs and expected results", t, func() {
 		enabledURIOnly := EnabledOptions{false, false, false, true}
 		testCases := []uriTester{
@@ -420,7 +432,7 @@ func TestParseAndSetOptions(t *testing.T) {
 				ShouldError: false,
 			},
 			{
-				Name: "should ask for password",
+				Name: "should ask for user password",
 				CS: connstring.ConnString{
 					AuthMechanism: "MONGODB-X509",
 					AuthSource:    "",
@@ -452,8 +464,8 @@ func TestParseAndSetOptions(t *testing.T) {
 					Kerberos:       &Kerberos{},
 					enabledOptions: EnabledOptions{Connection: true, URI: true},
 				},
-				ShouldError:          false,
-				ShouldAskForPassword: true,
+				ShouldError:              false,
+				AuthShouldAskForPassword: true,
 			},
 			{
 				Name: "single connect sets 'Direct'",
@@ -515,6 +527,27 @@ func TestParseAndSetOptions(t *testing.T) {
 				},
 				ShouldError: false,
 			},
+			{
+				Name: "RetryWrites is set when CS contains it",
+				CS: connstring.ConnString{
+					RetryWritesSet: true,
+					RetryWrites:    false,
+				},
+				OptsIn: New("", "", "", "", true, enabledURIOnly),
+				OptsExpected: &ToolOptions{
+					General:        &General{},
+					Verbosity:      &Verbosity{},
+					Connection:     &Connection{},
+					URI:            &URI{},
+					SSL:            &SSL{},
+					Auth:           &Auth{},
+					Namespace:      &Namespace{},
+					Kerberos:       &Kerberos{},
+					enabledOptions: EnabledOptions{URI: true},
+					RetryWrites:    &FalseValue,
+				},
+				ShouldError: false,
+			},
 			{
 				Name: "Direct is false when loadbalanced == true",
 				CS: connstring.ConnString{
@@ -608,7 +641,8 @@ func TestParseAndSetOptions(t *testing.T) {
 				So(testCase.OptsIn.SSL.UseSSL, ShouldResemble, testCase.OptsExpected.SSL.UseSSL)
 				So(testCase.OptsIn.Kerberos.Service, ShouldResemble, testCase.OptsExpected.Kerberos.Service)
 				So(testCase.OptsIn.Kerberos.ServiceHost, ShouldResemble, testCase.OptsExpected.Kerberos.ServiceHost)
-				So(testCase.OptsIn.Auth.ShouldAskForPassword(), ShouldEqual, testCase.OptsIn.ShouldAskForPassword())
+				So(testCase.OptsIn.RetryWrites, ShouldResemble, testCase.OptsExpected.RetryWrites)
+				So(testCase.OptsIn.Auth.ShouldAskForPassword(), ShouldEqual, testCase.OptsExpected.Auth.ShouldAskForPassword())
 			}
 		})
 	})
@@ -908,8 +942,8 @@ func TestOptionsParsing(t *testing.T) {
 			{"--sslCAFile", "sslCertificateAuthorityFile", "foo", "bar"},
 			{"--sslCAFile", "tlsCAFile", "foo", "bar"},
 
-			{"--sslPEMKeyFile", "sslClientCertificateKeyFile", "foo", "bar"},
-			{"--sslPEMKeyFile", "tlsCertificateKeyFile", "foo", "bar"},
+			{"--sslPEMKeyFile", "sslClientCertificateKeyFile", "../db/testdata/test-client-pkcs8-unencrypted.pem", "bar"},
+			{"--sslPEMKeyFile", "tlsCertificateKeyFile", "../db/testdata/test-client-pkcs8-unencrypted.pem", "bar"},
 
 			{"--sslPEMKeyPassword", "sslClientCertificateKeyPassword", "foo", "bar"},
 			{"--sslPEMKeyPassword", "tlsCertificateKeyFilePassword", "foo", "bar"},
@@ -968,11 +1002,11 @@ func TestOptionsParsingForSRV(t *testing.T) {
 	testtype.SkipUnlessTestType(t, testtype.SRVConnectionStringTestType)
 	atlasURI, ok := os.LookupEnv("ATLAS_URI")
 	if !ok {
-		t.Errorf("test requires ATLAS_URI to be set")
+		t.Fatalf("test requires ATLAS_URI to be set")
 	}
 	cs, err := connstring.Parse(atlasURI)
 	if err != nil {
-		t.Errorf("Error parsing ATLAS_URI: %s", err)
+		t.Fatalf("Failed to parse ATLAS_URI (%s): %s", atlasURI, err)
 	}
 
 	Convey("With a list of CLI options and URIs parsing should succeed or fail as expected", t, func() {
@@ -999,7 +1033,7 @@ func TestHiddenOptionsDefaults(t *testing.T) {
 	Convey("With a ToolOptions parsed", t, func() {
 		enabled := EnabledOptions{Connection: true}
 		opts := New("", "", "", "", true, enabled)
-		_, err := opts.parser.ParseArgs([]string{})
+		_, err := opts.CallArgParser([]string{})
 		So(err, ShouldBeNil)
 		Convey("hidden options should have expected values", func() {
 			So(opts.MaxProcs, ShouldEqual, runtime.NumCPU())
@@ -1059,3 +1093,135 @@ func TestDeprecationWarning(t *testing.T) {
 		})
 	})
 }
+
+func TestPasswordPrompt(t *testing.T) {
+	testtype.SkipUnlessTestType(t, testtype.UnitTestType)
+
+	pw := "some-password"
+	expectPrompt := regexp.MustCompile(`.*password.*:\n`)
+
+	t.Run("no prompt with user unset", func(t *testing.T) {
+		stderr, cleanupStderr := mockStderr(t)
+		defer cleanupStderr()
+
+		cleanup := mockStdin(t, pw)
+		defer cleanup()
+
+		opts := newTestOpts(t)
+		err := opts.NormalizeOptionsAndURI()
+		require.NoError(t, err)
+
+		prompt, err := ioutil.ReadFile(stderr.Name())
+		require.NoError(t, err)
+		require.Empty(t, string(prompt))
+	})
+
+	t.Run("prompt when user is set and password is not", func(t *testing.T) {
+		stderr, cleanupStderr := mockStderr(t)
+		defer cleanupStderr()
+
+		cleanup := mockStdin(t, pw)
+		defer cleanup()
+
+		opts := newTestOpts(t)
+		opts.Auth.Username = "someuser"
+		err := opts.NormalizeOptionsAndURI()
+		require.NoError(t, err)
+
+		prompt, err := ioutil.ReadFile(stderr.Name())
+		require.NoError(t, err)
+		require.Regexp(t, expectPrompt, string(prompt))
+		require.Equal(t, pw, opts.ConnString.Password)
+	})
+
+	t.Run("prompt when host, port, and user are set but uri is not", func(t *testing.T) {
+		stderr, cleanupStderr := mockStderr(t)
+		defer cleanupStderr()
+
+		cleanup := mockStdin(t, pw)
+		defer cleanup()
+
+		opts := newTestOpts(t)
+		opts.Auth.Username = "someuser"
+		opts.Host = "localhost"
+		opts.Port = "12345"
+		opts.URI = nil
+		err := opts.NormalizeOptionsAndURI()
+		require.NoError(t, err)
+
+		prompt, err := ioutil.ReadFile(stderr.Name())
+		require.NoError(t, err)
+		require.Regexp(t, expectPrompt, string(prompt))
+		require.Equal(t, pw, opts.ConnString.Password)
+	})
+
+	t.Run("prompt when user is set and password is not and mechanism is PLAIN", func(t *testing.T) {
+		stderr, cleanupStderr := mockStderr(t)
+		defer cleanupStderr()
+
+		cleanup := mockStdin(t, pw)
+		defer cleanup()
+
+		opts := newTestOpts(t)
+		opts.Auth.Username = "someuser"
+		opts.Auth.Mechanism = "PLAIN"
+		opts.SSL.UseSSL = true
+		err := opts.NormalizeOptionsAndURI()
+		require.NoError(t, err)
+
+		prompt, err := ioutil.ReadFile(stderr.Name())
+		require.NoError(t, err)
+		require.Regexp(t, expectPrompt, string(prompt))
+		require.Equal(t, pw, opts.ConnString.Password)
+	})
+}
+
+func newTestOpts(t *testing.T) *ToolOptions {
+	enabled := EnabledOptions{Auth: true, Connection: true, URI: true}
+	opts := New("test", "", "", "", true, enabled)
+
+	var err error
+	opts.URI, err = NewURI("mongodb://localhost:12345")
+	require.NoError(t, err)
+
+	return opts
+}
+
+func mockStderr(t *testing.T) (*os.File, func()) {
+	file, err := os.CreateTemp("", "mongo-tools-mock-stderr")
+	require.NoError(t, err)
+
+	oldStderr := os.Stderr
+	os.Stderr = file
+
+	return file, func() {
+		os.Stderr = oldStderr
+		file.Close()
+		rmErr := os.Remove(file.Name())
+		require.NoError(t, rmErr)
+	}
+}
+
+func mockStdin(t *testing.T, content string) func() {
+	file, err := os.CreateTemp("", "mongo-tools-mock-stdin")
+	require.NoError(t, err)
+
+	_, err = file.WriteString(content)
+	require.NoError(t, err)
+
+	err = file.Close()
+	require.NoError(t, err)
+
+	oldStdin := os.Stdin
+
+	file, err = os.Open(file.Name())
+	require.NoError(t, err)
+	os.Stdin = file
+
+	return func() {
+		os.Stdin = oldStdin
+		file.Close()
+		rmErr := os.Remove(file.Name())
+		require.NoError(t, rmErr)
+	}
+}
