@@ -18,10 +18,14 @@ import (
 	"strings"
 
 	"github.com/mongodb/mongo-tools/common/archive"
+	"github.com/mongodb/mongo-tools/common/bsonutil"
 	"github.com/mongodb/mongo-tools/common/db"
+	"github.com/mongodb/mongo-tools/common/dumprestore"
 	"github.com/mongodb/mongo-tools/common/intents"
 	"github.com/mongodb/mongo-tools/common/log"
 	"github.com/mongodb/mongo-tools/common/util"
+
+	"golang.org/x/exp/slices"
 )
 
 type NilPos struct{}
@@ -142,7 +146,13 @@ func (f *stdoutFile) Close() error {
 // shouldSkipSystemNamespace returns true when a namespace (database +
 // collection name) match certain reserved system namespaces that must
 // not be dumped.
-func shouldSkipSystemNamespace(dbName, collName string) bool {
+// By default dumping the entire cluster will only dump config collections
+// in dumprestore.ConfigCollectionsToKeep. Every other config collection is ignoered.
+// If you set --db=config then everything is included.
+// If you set --db=config --collection=foo, then shouldSkipSystemNamespace() is
+// never hit since CreateCollectionIntent() is run directly. In this case
+// config.foo will be the olny collection dumped.
+func (dump *MongoDump) shouldSkipSystemNamespace(dbName, collName string) bool {
 	// ignore <db>.system.* except for admin; ignore other specific
 	// collections in config and admin databases used for 3.6 features.
 	switch dbName {
@@ -151,15 +161,10 @@ func shouldSkipSystemNamespace(dbName, collName string) bool {
 			return true
 		}
 	case "config":
-		if collName == "transactions" ||
-			collName == "system.sessions" ||
-			collName == "transaction_coordinators" ||
-			collName == "system.indexBuilds" ||
-			collName == "image_collection" ||
-			collName == "mongos" ||
-			strings.HasPrefix(collName, "cache.") {
-			return true
+		if dump.ToolOptions.DB == "config" {
+			return false
 		}
+		return !slices.Contains(dumprestore.ConfigCollectionsToKeep, collName)
 	default:
 		if collName == "system.js" {
 			return false
@@ -350,8 +355,8 @@ func (dump *MongoDump) NewIntentFromOptions(dbName string, ci *db.CollectionInfo
 		}
 
 		if dump.OutputOptions.ViewsAsCollections && ci.IsView() {
-			delete(intent.Options, "viewOn")
-			delete(intent.Options, "pipeline")
+			bsonutil.RemoveKey("viewOn", &intent.Options)
+			bsonutil.RemoveKey("pipeline", &intent.Options)
 		}
 		//Set the MetadataFile path.
 		if dump.OutputOptions.Archive != "" {
@@ -408,13 +413,18 @@ func (dump *MongoDump) CreateIntentsForDatabase(dbName string) error {
 		if err != nil {
 			return fmt.Errorf("error decoding collection info: %v", err)
 		}
-		if shouldSkipSystemNamespace(dbName, collInfo.Name) {
-			log.Logvf(log.DebugHigh, "will not dump system collection '%s.%s'", dbName, collInfo.Name)
-			continue
-		}
+
+		// This MUST precede the remaining checks since it avoids
+		// a mid-reshard backup.
 		if dbName == "config" && dump.OutputOptions.Oplog && isReshardingCollection(collInfo.Name) {
 			return fmt.Errorf("detected resharding in progress. Cannot dump with --oplog while resharding")
 		}
+
+		if dump.shouldSkipSystemNamespace(dbName, collInfo.Name) {
+			log.Logvf(log.DebugHigh, "will not dump system collection '%s.%s'", dbName, collInfo.Name)
+			continue
+		}
+
 		if dump.shouldSkipCollection(collInfo.Name) {
 			log.Logvf(log.DebugLow, "skipping dump of %v.%v, it is excluded", dbName, collInfo.Name)
 			continue
@@ -433,19 +443,40 @@ func (dump *MongoDump) CreateIntentsForDatabase(dbName string) error {
 	return colsIter.Err()
 }
 
-// CreateAllIntents iterates through all dbs and collections and builds
-// dump intents for each collection.
-func (dump *MongoDump) CreateAllIntents() error {
+func (dump *MongoDump) GetValidDbs() ([]string, error) {
+	var validDbs []string
+	dump.SessionProvider.GetSession()
 	dbs, err := dump.SessionProvider.DatabaseNames()
 	if err != nil {
-		return fmt.Errorf("error getting database names: %v", err)
+		return nil, fmt.Errorf("error getting database names: %v", err)
 	}
 	log.Logvf(log.DebugHigh, "found databases: %v", strings.Join(dbs, ", "))
+
 	for _, dbName := range dbs {
 		if dbName == "local" {
 			// local can only be explicitly dumped
 			continue
 		}
+		if dbName == "admin" && dump.isAtlasProxy {
+			// admin can't be dumped if the cluster is connected via atlas proxy
+			continue
+		}
+
+		validDbs = append(validDbs, dbName)
+	}
+
+	return validDbs, nil
+}
+
+// CreateAllIntents iterates through all dbs and collections and builds
+// dump intents for each collection. Returns the db names that the intents
+// are created from.
+func (dump *MongoDump) CreateAllIntents() error {
+	dbs, err := dump.GetValidDbs()
+	if err != nil {
+		return err
+	}
+	for _, dbName := range dbs {
 		if err := dump.CreateIntentsForDatabase(dbName); err != nil {
 			return fmt.Errorf("error creating intents for database %s: %v", dbName, err)
 		}
