@@ -1,17 +1,24 @@
+// Copyright (C) MongoDB, Inc. 2014-present.
+//
+// Licensed under the Apache License, Version 2.0 (the "License"); you may
+// not use this file except in compliance with the License. You may obtain
+// a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+
 package mongorestore
 
 import (
+	"encoding/hex"
 	"fmt"
+	"strings"
 
-	"github.com/mongodb/mongo-tools/common"
-	"github.com/mongodb/mongo-tools/common/bsonutil"
 	"github.com/mongodb/mongo-tools/common/db"
+	"github.com/mongodb/mongo-tools/common/idx"
 	"github.com/mongodb/mongo-tools/common/intents"
-	"github.com/mongodb/mongo-tools/common/json"
 	"github.com/mongodb/mongo-tools/common/log"
 	"github.com/mongodb/mongo-tools/common/util"
-	"gopkg.in/mgo.v2"
-	"gopkg.in/mgo.v2/bson"
+	"go.mongodb.org/mongo-driver/bson"
+	"go.mongodb.org/mongo-driver/bson/primitive"
+	"go.mongodb.org/mongo-driver/mongo"
 )
 
 // Specially treated restore collection types.
@@ -30,69 +37,28 @@ type authVersionPair struct {
 
 // Metadata holds information about a collection's options and indexes.
 type Metadata struct {
-	Options bson.D          `json:"options,omitempty"`
-	Indexes []IndexDocument `json:"indexes"`
-}
-
-// this struct is used to read in the options of a set of indexes
-type metaDataMapIndex struct {
-	Indexes []bson.M `json:"indexes"`
-}
-
-// IndexDocument holds information about a collection's index.
-type IndexDocument struct {
-	Options bson.M `bson:",inline"`
-	Key     bson.D `bson:"key"`
+	Options        bson.D               `bson:"options,omitempty"`
+	Indexes        []*idx.IndexDocument `bson:"indexes"`
+	UUID           string               `bson:"uuid"`
+	CollectionName string               `bson:"collectionName"`
 }
 
 // MetadataFromJSON takes a slice of JSON bytes and unmarshals them into usable
 // collection options and indexes for restoring collections.
-func (restore *MongoRestore) MetadataFromJSON(jsonBytes []byte) (bson.D, []IndexDocument, error) {
+func (restore *MongoRestore) MetadataFromJSON(jsonBytes []byte) (*Metadata, error) {
 	if len(jsonBytes) == 0 {
 		// skip metadata parsing if the file is empty
-		return nil, nil, nil
+		return nil, nil
 	}
 
 	meta := &Metadata{}
 
-	err := json.Unmarshal(jsonBytes, meta)
+	err := bson.UnmarshalExtJSON(jsonBytes, true, meta)
 	if err != nil {
-		return nil, nil, err
+		return nil, err
 	}
 
-	// first get the ordered key information for each index,
-	// then merge it with a set of options stored as a map
-	metaAsMap := metaDataMapIndex{}
-	err = json.Unmarshal(jsonBytes, &metaAsMap)
-	if err != nil {
-		return nil, nil, fmt.Errorf("error unmarshalling metadata as map: %v", err)
-	}
-	for i := range meta.Indexes {
-		// remove "key" from the map so we can decode it properly later
-		delete(metaAsMap.Indexes[i], "key")
-
-		// parse extra index fields
-		meta.Indexes[i].Options = metaAsMap.Indexes[i]
-		if err := bsonutil.ConvertJSONDocumentToBSON(meta.Indexes[i].Options); err != nil {
-			return nil, nil, fmt.Errorf("extended json error: %v", err)
-		}
-
-		// parse the values of the index keys, so we can support extended json
-		for pos, field := range meta.Indexes[i].Key {
-			meta.Indexes[i].Key[pos].Value, err = bsonutil.ParseJSONValue(field.Value)
-			if err != nil {
-				return nil, nil, fmt.Errorf("extended json in '%v' field: %v", field.Name, err)
-			}
-		}
-	}
-
-	// parse the values of options fields, to support extended json
-	meta.Options, err = bsonutil.GetExtendedBsonD(meta.Options)
-	if err != nil {
-		return nil, nil, fmt.Errorf("extended json in 'options': %v", err)
-	}
-
-	return meta.Options, meta.Indexes, nil
+	return meta, nil
 }
 
 // LoadIndexesFromBSON reads indexes from the index BSON files and
@@ -113,11 +79,14 @@ func (restore *MongoRestore) LoadIndexesFromBSON() error {
 		defer bsonSource.Close()
 
 		// iterate over stored indexes, saving all that match the collection
-		indexDocument := &IndexDocument{}
-		for bsonSource.Next(indexDocument) {
+		for {
+			indexDocument := &idx.IndexDocument{}
+			if !bsonSource.Next(&indexDocument) {
+				break
+			}
 			namespace := indexDocument.Options["ns"].(string)
 			dbCollectionIndexes[dbname][stripDBFromNS(namespace)] =
-				append(dbCollectionIndexes[dbname][stripDBFromNS(namespace)], *indexDocument)
+				append(dbCollectionIndexes[dbname][stripDBFromNS(namespace)], indexDocument)
 		}
 		if err := bsonSource.Err(); err != nil {
 			return fmt.Errorf("error scanning system.indexes: %v", err)
@@ -128,12 +97,12 @@ func (restore *MongoRestore) LoadIndexesFromBSON() error {
 }
 
 func stripDBFromNS(ns string) string {
-	_, c := common.SplitNamespace(ns)
+	_, c := util.SplitNamespace(ns)
 	return c
 }
 
 // CollectionExists returns true if the given intent's collection exists.
-func (restore *MongoRestore) CollectionExists(intent *intents.Intent) (bool, error) {
+func (restore *MongoRestore) CollectionExists(dbName, coll string) (bool, error) {
 	restore.knownCollectionsMutex.Lock()
 	defer restore.knownCollectionsMutex.Unlock()
 
@@ -143,43 +112,62 @@ func (restore *MongoRestore) CollectionExists(intent *intents.Intent) (bool, err
 	}
 
 	// first check if we haven't done listCollections for this database already
-	if restore.knownCollections[intent.DB] == nil {
+	if restore.knownCollections[dbName] == nil {
 		// if the database name isn't in the cache, grab collection
 		// names from the server
 		session, err := restore.SessionProvider.GetSession()
 		if err != nil {
 			return false, fmt.Errorf("error establishing connection: %v", err)
 		}
-		defer session.Close()
-		collections, err := session.DB(intent.DB).CollectionNames()
+		collections, err := session.Database(dbName).ListCollections(nil, bson.M{})
 		if err != nil {
 			return false, err
 		}
 		// update the cache
-		restore.knownCollections[intent.DB] = collections
+		for collections.Next(nil) {
+			colNameRaw := collections.Current.Lookup("name")
+			colName, ok := colNameRaw.StringValueOK()
+			if !ok {
+				return false, fmt.Errorf("invalid collection name: %v", colNameRaw)
+			}
+			restore.knownCollections[dbName] = append(restore.knownCollections[dbName], colName)
+		}
 	}
 
 	// now check the cache for the given collection name
-	exists := util.StringSliceContains(restore.knownCollections[intent.DB], intent.C)
+	exists := util.StringSliceContains(restore.knownCollections[dbName], coll)
 	return exists, nil
 }
 
+// addToKnownCollections will add a collection to the restore.knownCollections cache.
+// This is used to update the cache after a collection has been created during a restore.
+func (restore *MongoRestore) addToKnownCollections(intent *intents.Intent) {
+	restore.knownCollectionsMutex.Lock()
+	defer restore.knownCollectionsMutex.Unlock()
+
+	restore.knownCollections[intent.DB] = append(restore.knownCollections[intent.DB], intent.C)
+}
+
 // CreateIndexes takes in an intent and an array of index documents and
 // attempts to create them using the createIndexes command. If that command
 // fails, we fall back to individual index creation.
-func (restore *MongoRestore) CreateIndexes(intent *intents.Intent, indexes []IndexDocument) error {
+func (restore *MongoRestore) CreateIndexes(dbName string, collectionName string, indexes []*idx.IndexDocument) error {
 	// first, sanitize the indexes
+	var indexNames []string
 	for _, index := range indexes {
 		// update the namespace of the index before inserting
-		index.Options["ns"] = intent.Namespace()
+		index.Options["ns"] = dbName + "." + collectionName
 
 		// check for length violations before building the command
-		fullIndexName := fmt.Sprintf("%v.$%v", index.Options["ns"], index.Options["name"])
-		if len(fullIndexName) > 127 {
-			return fmt.Errorf(
-				"cannot restore index with namespace '%v': "+
-					"namespace is too long (max size is 127 bytes)", fullIndexName)
+		if restore.serverVersion.LT(db.Version{4, 2, 0}) {
+			fullIndexName := fmt.Sprintf("%v.$%v", index.Options["ns"], index.Options["name"])
+			if len(fullIndexName) > 127 {
+				return fmt.Errorf(
+					"cannot restore index with namespace '%v': "+
+						"namespace is too long (max size is 127 bytes)", fullIndexName)
+			}
 		}
+		indexNames = append(indexNames, index.Options["name"].(string))
 
 		// remove the index version, forcing an update,
 		// unless we specifically want to keep it
@@ -192,16 +180,20 @@ func (restore *MongoRestore) CreateIndexes(intent *intents.Intent, indexes []Ind
 	if err != nil {
 		return fmt.Errorf("error establishing connection: %v", err)
 	}
-	session.SetSafe(&mgo.Safe{})
-	defer session.Close()
 
 	// then attempt the createIndexes command
 	rawCommand := bson.D{
-		{"createIndexes", intent.C},
+		{"createIndexes", collectionName},
 		{"indexes", indexes},
 	}
-	results := bson.M{}
-	err = session.DB(intent.DB).Run(rawCommand, &results)
+
+	log.Logvf(log.Info, "\trun create Index command for indexes: %v", strings.Join(indexNames, ", "))
+
+	if restore.serverVersion.GTE(db.Version{4, 1, 9}) {
+		rawCommand = append(rawCommand, bson.E{"ignoreUnknownIndexOptions", true})
+	}
+
+	err = session.Database(dbName).RunCommand(nil, rawCommand).Err()
 	if err == nil {
 		return nil
 	}
@@ -213,7 +205,7 @@ func (restore *MongoRestore) CreateIndexes(intent *intents.Intent, indexes []Ind
 	log.Logv(log.Info, "\tcreateIndexes command not supported, attemping legacy index insertion")
 	for _, idx := range indexes {
 		log.Logvf(log.Info, "\tmanually creating index %v", idx.Options["name"])
-		err = restore.LegacyInsertIndex(intent, idx)
+		err = restore.LegacyInsertIndex(dbName, idx)
 		if err != nil {
 			return fmt.Errorf("error creating index %v: %v", idx.Options["name"], err)
 		}
@@ -223,17 +215,14 @@ func (restore *MongoRestore) CreateIndexes(intent *intents.Intent, indexes []Ind
 
 // LegacyInsertIndex takes in an intent and an index document and attempts to
 // create the index on the "system.indexes" collection.
-func (restore *MongoRestore) LegacyInsertIndex(intent *intents.Intent, index IndexDocument) error {
+func (restore *MongoRestore) LegacyInsertIndex(dbName string, index *idx.IndexDocument) error {
 	session, err := restore.SessionProvider.GetSession()
 	if err != nil {
 		return fmt.Errorf("error establishing connection: %v", err)
 	}
-	defer session.Close()
 
-	// overwrite safety to make sure we catch errors
-	session.SetSafe(&mgo.Safe{})
-	indexCollection := session.DB(intent.DB).C("system.indexes")
-	err = indexCollection.Insert(index)
+	indexCollection := session.Database(dbName).Collection("system.indexes")
+	_, err = indexCollection.InsertOne(nil, index)
 	if err != nil {
 		return fmt.Errorf("insert error: %v", err)
 	}
@@ -243,29 +232,113 @@ func (restore *MongoRestore) LegacyInsertIndex(intent *intents.Intent, index Ind
 
 // CreateCollection creates the collection specified in the intent with the
 // given options.
-func (restore *MongoRestore) CreateCollection(intent *intents.Intent, options bson.D) error {
-	command := append(bson.D{{"create", intent.C}}, options...)
-
+func (restore *MongoRestore) CreateCollection(intent *intents.Intent, options bson.D, uuid string) error {
 	session, err := restore.SessionProvider.GetSession()
 	if err != nil {
 		return fmt.Errorf("error establishing connection: %v", err)
 	}
-	defer session.Close()
 
-	res := bson.M{}
-	err = session.DB(intent.DB).Run(command, &res)
-	if err != nil {
+	switch {
+
+	case uuid != "":
+		return restore.createCollectionWithApplyOps(session, intent, options, uuid)
+	default:
+		return restore.createCollectionWithCommand(session, intent, options)
+	}
+
+}
+
+// UpdateAutoIndexId updates {autoIndexId: false} to {autoIndexId: true} if the server version is
+// >= 4.0 and the database is not `local`.
+func (restore *MongoRestore) UpdateAutoIndexId(options bson.D) {
+	if restore.serverVersion.GTE(db.Version{4, 0, 0}) {
+		for i, elem := range options {
+			if elem.Key == "autoIndexId" && elem.Value == false && restore.ToolOptions.Namespace.DB != "local" {
+				options[i].Value = true
+				log.Logvf(log.Always, "{autoIndexId: false} is not allowed in server versions >= 4.0. Changing to {autoIndexId: true}.")
+			}
+		}
+	}
+}
+
+func (restore *MongoRestore) createCollectionWithCommand(session *mongo.Client, intent *intents.Intent, options bson.D) error {
+	restore.UpdateAutoIndexId(options)
+
+	command := createCollectionCommand(intent, options)
+
+	// If there is no error, the result doesnt matter
+	singleRes := session.Database(intent.DB).RunCommand(nil, command, nil)
+	if err := singleRes.Err(); err != nil {
 		return fmt.Errorf("error running create command: %v", err)
 	}
+
+	res := bson.M{}
+	singleRes.Decode(&res)
 	if util.IsFalsy(res["ok"]) {
 		return fmt.Errorf("create command: %v", res["errmsg"])
 	}
 	return nil
+
+}
+
+func (restore *MongoRestore) createCollectionWithApplyOps(session *mongo.Client, intent *intents.Intent, options bson.D, uuidHex string) error {
+	restore.UpdateAutoIndexId(options)
+
+	command := createCollectionCommand(intent, options)
+	uuid, err := hex.DecodeString(uuidHex)
+	if err != nil {
+		return fmt.Errorf("Couldn't restore UUID because UUID was invalid: %s", err)
+	}
+
+	createOp := struct {
+		Operation string            `bson:"op"`
+		Namespace string            `bson:"ns"`
+		Object    bson.D            `bson:"o"`
+		UI        *primitive.Binary `bson:"ui,omitempty"`
+	}{
+		Operation: "c",
+		Namespace: intent.DB + ".$cmd",
+		Object:    command,
+		UI:        &primitive.Binary{Subtype: 0x04, Data: uuid},
+	}
+
+	return restore.ApplyOps(session, []interface{}{createOp})
+}
+
+func createCollectionCommand(intent *intents.Intent, options bson.D) bson.D {
+	return append(bson.D{{"create", intent.C}}, options...)
 }
 
 // RestoreUsersOrRoles accepts a users intent and a roles intent, and restores
 // them via _mergeAuthzCollections. Either or both can be nil. In the latter case
 // nothing is done.
+//
+// _mergeAuthzCollections is an internal server command implemented specifically for mongorestore. Instead of inserting
+// into the admin.system.{roles, users} collections (which isn't allowed due to some locking policies), we construct
+// temporary collections that are then merged with or replace the existing ones.
+//
+// The "drop" argument that determines whether the merge replaces the existing users/roles or adds to them.
+//
+// The "db" argument determines which databases' users and roles are merged. If left blank, it merges users and roles
+// from all databases. When the user restores the admin database, we assume they wish to restore the users and roles for
+// all databases, not just the admin ones, so we leave the "db" field blank in that case.
+//
+// The "temp{Users,Roles}Collection" arguments determine which temporary collections to merge from, and the presence of
+// either determines which collection to merge to. (e.g. if tempUsersCollection is defined, admin.system.users is merged
+// into).
+//
+// This command must be run on the "admin" database. Thus, the temporary collections must be on the admin db as well.
+// This command must also be run on the primary.
+//
+// Example command:
+//
+//	{
+//	   _mergeAuthzCollections: 1,
+//	   db: "foo",
+//	   tempUsersCollection: "myTempUsers"
+//	   drop: true
+//	   writeConcern: {w: "majority"}
+//	}
 func (restore *MongoRestore) RestoreUsersOrRoles(users, roles *intents.Intent) error {
 
 	type loopArg struct {
@@ -298,7 +371,6 @@ func (restore *MongoRestore) RestoreUsersOrRoles(users, roles *intents.Intent) e
 	if err != nil {
 		return fmt.Errorf("error establishing connection: %v", err)
 	}
-	defer session.Close()
 
 	// For each of the users and roles intents:
 	//   build up the mergeArgs component of the _mergeAuthzCollections command
@@ -311,8 +383,9 @@ func (restore *MongoRestore) RestoreUsersOrRoles(users, roles *intents.Intent) e
 			log.Logvf(log.Always, "%v file '%v' is empty; skipping %v restoration", arg.intentType, arg.intent.Location, arg.intentType)
 		}
 		log.Logvf(log.Always, "restoring %v from %v", arg.intentType, arg.intent.Location)
-		mergeArgs = append(mergeArgs, bson.DocElem{
-			Name:  arg.mergeParamName,
+
+		mergeArgs = append(mergeArgs, bson.E{
+			Key:   arg.mergeParamName,
 			Value: "admin." + arg.tempCollectionName,
 		})
 
@@ -324,38 +397,38 @@ func (restore *MongoRestore) RestoreUsersOrRoles(users, roles *intents.Intent) e
 		bsonSource := db.NewDecodedBSONSource(db.NewBSONSource(arg.intent.BSONFile))
 		defer bsonSource.Close()
 
-		tempCollectionNameExists, err := restore.CollectionExists(&intents.Intent{DB: "admin", C: arg.tempCollectionName})
+		tempCollectionNameExists, err := restore.CollectionExists("admin", arg.tempCollectionName)
 		if err != nil {
 			return err
 		}
 		if tempCollectionNameExists {
 			log.Logvf(log.Info, "dropping preexisting temporary collection admin.%v", arg.tempCollectionName)
-			err = session.DB("admin").C(arg.tempCollectionName).DropCollection()
+			err = session.Database("admin").Collection(arg.tempCollectionName).Drop(nil)
 			if err != nil {
 				return fmt.Errorf("error dropping preexisting temporary collection %v: %v", arg.tempCollectionName, err)
 			}
 		}
 
 		log.Logvf(log.DebugLow, "restoring %v to temporary collection", arg.intentType)
-		if _, err = restore.RestoreCollectionToDB("admin", arg.tempCollectionName, bsonSource, arg.intent.BSONFile, 0); err != nil {
-			return fmt.Errorf("error restoring %v: %v", arg.intentType, err)
+		result := restore.RestoreCollectionToDB("admin", arg.tempCollectionName, bsonSource, arg.intent.BSONFile, 0, "")
+		if result.Err != nil {
+			return fmt.Errorf("error restoring %v: %v", arg.intentType, result.Err)
 		}
 
 		// make sure we always drop the temporary collection
-		defer func() {
+		defer func(cleanupArg loopArg) {
 			session, e := restore.SessionProvider.GetSession()
 			if e != nil {
 				// logging errors here because this has no way of returning that doesn't mask other errors
-				log.Logvf(log.Info, "error establishing connection to drop temporary collection admin.%v: %v", arg.tempCollectionName, e)
+				log.Logvf(log.Info, "error establishing connection to drop temporary collection admin.%v: %v", cleanupArg.tempCollectionName, e)
 				return
 			}
-			defer session.Close()
-			log.Logvf(log.DebugHigh, "dropping temporary collection admin.%v", arg.tempCollectionName)
-			e = session.DB("admin").C(arg.tempCollectionName).DropCollection()
+			log.Logvf(log.DebugHigh, "dropping temporary collection admin.%v", cleanupArg.tempCollectionName)
+			e = session.Database("admin").Collection(cleanupArg.tempCollectionName).Drop(nil)
 			if e != nil {
-				log.Logvf(log.Info, "error dropping temporary collection admin.%v: %v", arg.tempCollectionName, e)
+				log.Logvf(log.Info, "error dropping temporary collection admin.%v: %v", cleanupArg.tempCollectionName, e)
 			}
-		}()
+		}(arg)
 		userTargetDB = arg.intent.DB
 	}
 
@@ -364,34 +437,39 @@ func (restore *MongoRestore) RestoreUsersOrRoles(users, roles *intents.Intent) e
 		userTargetDB = ""
 	}
 
-	// we have to manually convert mgo's safety to a writeconcern object
-	writeConcern := bson.M{}
-	if restore.safety == nil {
-		writeConcern["w"] = 0
-	} else {
-		if restore.safety.WMode != "" {
-			writeConcern["w"] = restore.safety.WMode
-		} else {
-			writeConcern["w"] = restore.safety.W
-		}
-	}
+	adminDB := session.Database("admin")
 
-	command := bsonutil.MarshalD{}
+	command := bson.D{}
 	command = append(command,
-		bson.DocElem{Name: "_mergeAuthzCollections", Value: 1})
+		bson.E{Key: "_mergeAuthzCollections", Value: 1})
 	command = append(command,
 		mergeArgs...)
 	command = append(command,
-		bson.DocElem{Name: "drop", Value: restore.OutputOptions.Drop},
-		bson.DocElem{Name: "writeConcern", Value: writeConcern},
-		bson.DocElem{Name: "db", Value: userTargetDB})
+		bson.E{Key: "drop", Value: restore.OutputOptions.Drop},
+		bson.E{Key: "db", Value: userTargetDB})
+
+	if restore.ToolOptions.WriteConcern != nil {
+		_, wcBson, err := restore.ToolOptions.WriteConcern.MarshalBSONValue()
+		if err != nil {
+			return fmt.Errorf("error parsing write concern: %v", err)
+		}
+
+		writeConcern := bson.M{}
+		err = bson.Unmarshal(wcBson, &writeConcern)
+		if err != nil {
+			return fmt.Errorf("error parsing write concern: %v", err)
+		}
+
+		command = append(command, bson.E{Key: "writeConcern", Value: writeConcern})
+	}
 
 	log.Logvf(log.DebugLow, "merging users/roles from temp collections")
-	res := bson.M{}
-	err = session.Run(command, &res)
-	if err != nil {
+	resSingle := adminDB.RunCommand(nil, command)
+	if err = resSingle.Err(); err != nil {
 		return fmt.Errorf("error running merge command: %v", err)
 	}
+	res := bson.M{}
+	resSingle.Decode(&res)
 	if util.IsFalsy(res["ok"]) {
 		return fmt.Errorf("_mergeAuthzCollections command: %v", res["errmsg"])
 	}
@@ -410,7 +488,7 @@ func (restore *MongoRestore) GetDumpAuthVersion() (int, error) {
 			// If we are using --restoreDbUsersAndRoles, we cannot guarantee an
 			// $admin.system.version collection from a 2.6 server,
 			// so we can assume up to version 3.
-			log.Logvf(log.Always, "no system.version bson file found in '%v' database dump", restore.NSOptions.DB)
+			log.Logvf(log.Always, "no system.version bson file found in '%v' database dump", restore.ToolOptions.Namespace.DB)
 			log.Logv(log.Always, "warning: assuming users and roles collections are of auth version 3")
 			log.Logv(log.Always, "if users are from an earlier version of MongoDB, they may not restore properly")
 			return 3, nil
@@ -428,15 +506,24 @@ func (restore *MongoRestore) GetDumpAuthVersion() (int, error) {
 	bsonSource := db.NewDecodedBSONSource(db.NewBSONSource(intent.BSONFile))
 	defer bsonSource.Close()
 
-	versionDoc := bson.M{}
-	for bsonSource.Next(&versionDoc) {
+	for {
+		versionDoc := bson.M{}
+		if !bsonSource.Next(&versionDoc) {
+			break
+		}
+
 		id, ok := versionDoc["_id"].(string)
 		if ok && id == "authSchema" {
-			authVersion, ok := versionDoc["currentVersion"].(int)
-			if ok {
+			switch authVersion := versionDoc["currentVersion"].(type) {
+			case int:
 				return authVersion, nil
+			case int32:
+				return int(authVersion), nil
+			case int64:
+				return int(authVersion), nil
+			default:
+				return 0, fmt.Errorf("can't unmarshal system.version curentVersion as an int: %v", versionDoc["currentVersion"])
 			}
-			return 0, fmt.Errorf("can't unmarshal system.version curentVersion as an int")
 		}
 		log.Logvf(log.DebugLow, "system.version document is not an authSchema %v", versionDoc["_id"])
 	}
@@ -499,13 +586,19 @@ func (restore *MongoRestore) ValidateAuthVersions() error {
 // ShouldRestoreUsersAndRoles returns true if mongorestore should go through
 // through the process of restoring collections pertaining to authentication.
 func (restore *MongoRestore) ShouldRestoreUsersAndRoles() bool {
+	if restore.SkipUsersAndRoles {
+		return false
+	}
+
 	// If the user has done anything that would indicate the restoration
 	// of users and roles (i.e. used --restoreDbUsersAndRoles, -d admin, or
-	// is doing a full restore), then we check if users or roles BSON files
-	// actually exist in the dump dir. If they do, return true.
-	if restore.InputOptions.RestoreDBUsersAndRoles ||
-		restore.NSOptions.DB == "" ||
-		restore.NSOptions.DB == "admin" {
+	// is doing a full restore), and the tool isn't connected to an atlas proxy
+	// then we check if users or roles BSON files actually exist in the dump
+	// dir. If they do, return true.
+	if (restore.InputOptions.RestoreDBUsersAndRoles ||
+		restore.ToolOptions.Namespace.DB == "" ||
+		restore.ToolOptions.Namespace.DB == "admin") &&
+		!restore.isAtlasProxy {
 		if restore.manager.Users() != nil || restore.manager.Roles() != nil {
 			return true
 		}
@@ -519,8 +612,7 @@ func (restore *MongoRestore) DropCollection(intent *intents.Intent) error {
 	if err != nil {
 		return fmt.Errorf("error establishing connection: %v", err)
 	}
-	defer session.Close()
-	err = session.DB(intent.DB).C(intent.C).DropCollection()
+	err = session.Database(intent.DB).Collection(intent.C).Drop(nil)
 	if err != nil {
 		return fmt.Errorf("error dropping collection: %v", err)
 	}
