@@ -1,3 +1,9 @@
+// Copyright (C) MongoDB, Inc. 2014-present.
+//
+// Licensed under the Apache License, Version 2.0 (the "License"); you may
+// not use this file except in compliance with the License. You may obtain
+// a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+
 // Package mongorestore writes BSON data to a MongoDB instance.
 package mongorestore
 
@@ -8,19 +14,32 @@ import (
 	"io/ioutil"
 	"os"
 	"path/filepath"
+	"strings"
 	"sync"
+	"time"
 
 	"github.com/mongodb/mongo-tools/common/archive"
 	"github.com/mongodb/mongo-tools/common/auth"
 	"github.com/mongodb/mongo-tools/common/db"
+	"github.com/mongodb/mongo-tools/common/idx"
 	"github.com/mongodb/mongo-tools/common/intents"
 	"github.com/mongodb/mongo-tools/common/log"
 	"github.com/mongodb/mongo-tools/common/options"
 	"github.com/mongodb/mongo-tools/common/progress"
 	"github.com/mongodb/mongo-tools/common/util"
 	"github.com/mongodb/mongo-tools/mongorestore/ns"
-	"gopkg.in/mgo.v2"
-	"gopkg.in/mgo.v2/bson"
+	"go.mongodb.org/mongo-driver/bson/primitive"
+)
+
+const (
+	progressBarLength   = 24
+	progressBarWaitTime = time.Second * 3
+)
+
+const (
+	deprecatedDBAndCollectionsOptionsWarning = "The --db and --collection flags are deprecated for " +
+		"this use-case; please use --nsInclude instead, " +
+		"i.e. with --nsInclude=${DATABASE}.${COLLECTION}"
 )
 
 // MongoRestore is a container for the user-specified options and
@@ -36,13 +55,16 @@ type MongoRestore struct {
 
 	TargetDirectory string
 
+	// Skip restoring users and roles, regardless of namespace, when true.
+	SkipUsersAndRoles bool
+
 	// other internal state
 	manager *intents.Manager
-	safety  *mgo.Safe
 
 	objCheck         bool
-	oplogLimit       bson.MongoTimestamp
+	oplogLimit       primitive.Timestamp
 	isMongos         bool
+	isAtlasProxy     bool
 	useWriteCommands bool
 	authVersions     authVersionPair
 
@@ -57,16 +79,75 @@ type MongoRestore struct {
 	// indexes belonging to dbs and collections
 	dbCollectionIndexes map[string]collectionIndexes
 
+	indexCatalog *idx.IndexCatalog
+
 	archive *archive.Reader
 
-	// channel on which to notify if/when a termination signal is received
-	termChan chan struct{}
+	// boolean set if termination signal received; false by default
+	terminate bool
 
-	// for testing. If set, this value will be used instead of os.Stdin
-	stdin io.Reader
+	// Reader to take care of BSON input if not reading from the local filesystem.
+	// This is initialized to os.Stdin if unset.
+	InputReader io.Reader
+
+	// Server version for version-specific behavior
+	serverVersion db.Version
 }
 
-type collectionIndexes map[string][]IndexDocument
+type collectionIndexes map[string][]*idx.IndexDocument
+
+// New initializes an instance of MongoRestore according to the provided options.
+func New(opts Options) (*MongoRestore, error) {
+	provider, err := db.NewSessionProvider(*opts.ToolOptions)
+	if err != nil {
+		return nil, fmt.Errorf("error connecting to host: %v", err)
+	}
+
+	serverVersion, err := provider.ServerVersionArray()
+	if err != nil {
+		return nil, fmt.Errorf("error getting server version: %v", err)
+	}
+
+	// start up the progress bar manager
+	progressManager := progress.NewBarWriter(log.Writer(0), progressBarWaitTime, progressBarLength, true)
+	progressManager.Start()
+
+	restore := &MongoRestore{
+		ToolOptions:     opts.ToolOptions,
+		OutputOptions:   opts.OutputOptions,
+		InputOptions:    opts.InputOptions,
+		NSOptions:       opts.NSOptions,
+		TargetDirectory: opts.TargetDirectory,
+		SessionProvider: provider,
+		ProgressManager: progressManager,
+		serverVersion:   serverVersion,
+		terminate:       false,
+		indexCatalog:    idx.NewIndexCatalog(),
+	}
+
+	restore.isMongos, err = restore.SessionProvider.IsMongos()
+	if err != nil {
+		return nil, err
+	}
+	if restore.isMongos {
+		log.Logv(log.DebugLow, "restoring to a sharded system")
+	}
+	restore.isAtlasProxy = restore.SessionProvider.IsAtlasProxy()
+	if restore.isAtlasProxy {
+		log.Logv(log.DebugLow, "restoring to a MongoDB Atlas free or shared cluster")
+	}
+
+	return restore, nil
+}
+
+// Close ends any connections and cleans up other internal state.
+func (restore *MongoRestore) Close() {
+	restore.SessionProvider.Close()
+	barWriter, ok := restore.ProgressManager.(*progress.BarWriter)
+	if ok { // should always be ok
+		barWriter.Stop()
+	}
+}
 
 // ParseAndValidateOptions returns a non-nil error if user-supplied options are invalid.
 func (restore *MongoRestore) ParseAndValidateOptions() error {
@@ -81,36 +162,35 @@ func (restore *MongoRestore) ParseAndValidateOptions() error {
 		log.Logv(log.DebugHigh, "\tdumping with object check disabled")
 	}
 
-	if restore.NSOptions.DB == "" && restore.NSOptions.Collection != "" {
+	if restore.ToolOptions.Namespace.DB == "" && restore.ToolOptions.Namespace.Collection != "" {
 		return fmt.Errorf("cannot restore a collection without a specified database")
 	}
 
-	if restore.NSOptions.DB != "" {
-		if err := util.ValidateDBName(restore.NSOptions.DB); err != nil {
+	if restore.ToolOptions.Namespace.DB != "" {
+		if err := util.ValidateDBName(restore.ToolOptions.Namespace.DB); err != nil {
 			return fmt.Errorf("invalid db name: %v", err)
 		}
 	}
-	if restore.NSOptions.Collection != "" {
-		if err := util.ValidateCollectionGrammar(restore.NSOptions.Collection); err != nil {
+	if restore.ToolOptions.Namespace.Collection != "" {
+		if err := util.ValidateCollectionGrammar(restore.ToolOptions.Namespace.Collection); err != nil {
 			return fmt.Errorf("invalid collection name: %v", err)
 		}
 	}
-	if restore.InputOptions.RestoreDBUsersAndRoles && restore.NSOptions.DB == "" {
+	if restore.InputOptions.RestoreDBUsersAndRoles && restore.ToolOptions.Namespace.DB == "" {
 		return fmt.Errorf("cannot use --restoreDbUsersAndRoles without a specified database")
 	}
-	if restore.InputOptions.RestoreDBUsersAndRoles && restore.NSOptions.DB == "admin" {
+	if restore.InputOptions.RestoreDBUsersAndRoles && restore.ToolOptions.Namespace.DB == "admin" {
 		return fmt.Errorf("cannot use --restoreDbUsersAndRoles with the admin database")
 	}
 
-	var err error
-	restore.isMongos, err = restore.SessionProvider.IsMongos()
-	if err != nil {
-		return err
-	}
-	if restore.isMongos {
-		log.Logv(log.DebugLow, "restoring to a sharded system")
+	if restore.isAtlasProxy {
+		if restore.InputOptions.RestoreDBUsersAndRoles || restore.ToolOptions.Namespace.DB == "admin" {
+			return fmt.Errorf("cannot restore to the admin database when connected to a MongoDB Atlas free or shared cluster")
+		}
+		log.Logv(log.DebugLow, "restoring to a MongoDB Atlas free or shared cluster")
 	}
 
+	var err error
 	if restore.InputOptions.OplogLimit != "" {
 		if !restore.InputOptions.OplogReplay {
 			return fmt.Errorf("cannot use --oplogLimit without --oplogReplay enabled")
@@ -136,20 +216,11 @@ func (restore *MongoRestore) ParseAndValidateOptions() error {
 	}
 
 	log.Logvf(log.DebugLow, "connected to node type: %v", nodeType)
-	restore.safety, err = db.BuildWriteConcern(restore.OutputOptions.WriteConcern, nodeType,
-		restore.ToolOptions.URI.ParsedConnString())
-	if err != nil {
-		return fmt.Errorf("error parsing write concern: %v", err)
-	}
 
 	// deprecations with --nsInclude --nsExclude
-	if restore.NSOptions.DB != "" || restore.NSOptions.Collection != "" {
-		// these are only okay if restoring from a bson file
-		_, fileType := restore.getInfoFromFilename(restore.TargetDirectory)
-		if fileType != BSONFileType {
-			log.Logvf(log.Always, "the --db and --collection args should only be used when "+
-				"restoring from a BSON file. Other uses are deprecated and will not exist "+
-				"in the future; use --nsInclude instead")
+	if restore.ToolOptions.Namespace.DB != "" || restore.ToolOptions.Namespace.Collection != "" {
+		if filepath.Ext(restore.TargetDirectory) != ".bson" {
+			log.Logvf(log.Always, deprecatedDBAndCollectionsOptionsWarning)
 		}
 	}
 	if len(restore.NSOptions.ExcludedCollections) > 0 ||
@@ -158,7 +229,7 @@ func (restore *MongoRestore) ParseAndValidateOptions() error {
 			"are deprecated and will not exist in the future; use --nsExclude instead")
 	}
 	if restore.InputOptions.OplogReplay {
-		if len(restore.NSOptions.NSInclude) > 0 || restore.NSOptions.DB != "" {
+		if len(restore.NSOptions.NSInclude) > 0 || restore.ToolOptions.Namespace.DB != "" {
 			return fmt.Errorf("cannot use --oplogReplay with includes specified")
 		}
 		if len(restore.NSOptions.NSExclude) > 0 || len(restore.NSOptions.ExcludedCollections) > 0 ||
@@ -171,11 +242,11 @@ func (restore *MongoRestore) ParseAndValidateOptions() error {
 	}
 
 	includes := restore.NSOptions.NSInclude
-	if restore.NSOptions.DB != "" && restore.NSOptions.Collection != "" {
-		includes = append(includes, ns.Escape(restore.NSOptions.DB)+"."+
-			restore.NSOptions.Collection)
-	} else if restore.NSOptions.DB != "" {
-		includes = append(includes, ns.Escape(restore.NSOptions.DB)+".*")
+	if restore.ToolOptions.Namespace.DB != "" && restore.ToolOptions.Namespace.Collection != "" {
+		includes = append(includes, ns.Escape(restore.ToolOptions.Namespace.DB)+"."+
+			restore.ToolOptions.Namespace.Collection)
+	} else if restore.ToolOptions.Namespace.DB != "" {
+		includes = append(includes, ns.Escape(restore.ToolOptions.Namespace.DB)+".*")
 	}
 	if len(includes) == 0 {
 		includes = []string{"*"}
@@ -185,10 +256,10 @@ func (restore *MongoRestore) ParseAndValidateOptions() error {
 		return fmt.Errorf("invalid includes: %v", err)
 	}
 
-	if len(restore.NSOptions.ExcludedCollections) > 0 && restore.NSOptions.Collection != "" {
+	if len(restore.NSOptions.ExcludedCollections) > 0 && restore.ToolOptions.Namespace.Collection != "" {
 		return fmt.Errorf("--collection is not allowed when --excludeCollection is specified")
 	}
-	if len(restore.NSOptions.ExcludedCollectionPrefixes) > 0 && restore.NSOptions.Collection != "" {
+	if len(restore.NSOptions.ExcludedCollectionPrefixes) > 0 && restore.ToolOptions.Namespace.Collection != "" {
 		return fmt.Errorf("--collection is not allowed when --excludeCollectionsWithPrefix is specified")
 	}
 	excludes := restore.NSOptions.NSExclude
@@ -216,30 +287,39 @@ func (restore *MongoRestore) ParseAndValidateOptions() error {
 			"cannot specify a negative number of insertion workers per collection")
 	}
 
+	if restore.OutputOptions.MaintainInsertionOrder {
+		restore.OutputOptions.StopOnError = true
+		restore.OutputOptions.NumInsertionWorkers = 1
+	}
+
+	if restore.OutputOptions.PreserveUUID && !restore.OutputOptions.Drop {
+		return fmt.Errorf("cannot specify --preserveUUID without --drop")
+	}
+
 	// a single dash signals reading from stdin
 	if restore.TargetDirectory == "-" {
 		if restore.InputOptions.Archive != "" {
 			return fmt.Errorf(
 				"cannot restore from \"-\" when --archive is specified")
 		}
-		if restore.NSOptions.Collection == "" {
+		if restore.ToolOptions.Namespace.Collection == "" {
 			return fmt.Errorf("cannot restore from stdin without a specified collection")
 		}
 	}
-	if restore.stdin == nil {
-		restore.stdin = os.Stdin
+	if restore.InputReader == nil {
+		restore.InputReader = os.Stdin
 	}
 
 	return nil
 }
 
 // Restore runs the mongorestore program.
-func (restore *MongoRestore) Restore() error {
+func (restore *MongoRestore) Restore() Result {
 	var target archive.DirLike
 	err := restore.ParseAndValidateOptions()
 	if err != nil {
 		log.Logvf(log.DebugLow, "got error from options parsing: %v", err)
-		return err
+		return Result{Err: err}
 	}
 
 	// Build up all intents to be restored
@@ -252,23 +332,24 @@ func (restore *MongoRestore) Restore() error {
 		if restore.archive == nil {
 			archiveReader, err := restore.getArchiveReader()
 			if err != nil {
-				return err
+				return Result{Err: err}
 			}
 			restore.archive = &archive.Reader{
 				In:      archiveReader,
 				Prelude: &archive.Prelude{},
 			}
+			defer restore.archive.In.Close()
 		}
 		err = restore.archive.Prelude.Read(restore.archive.In)
 		if err != nil {
-			return err
+			return Result{Err: err}
 		}
 		log.Logvf(log.DebugLow, `archive format version "%v"`, restore.archive.Prelude.Header.FormatVersion)
 		log.Logvf(log.DebugLow, `archive server version "%v"`, restore.archive.Prelude.Header.ServerVersion)
 		log.Logvf(log.DebugLow, `archive tool version "%v"`, restore.archive.Prelude.Header.ToolVersion)
 		target, err = restore.archive.Prelude.NewPreludeExplorer()
 		if err != nil {
-			return err
+			return Result{Err: err}
 		}
 	} else if restore.TargetDirectory != "-" {
 		var usedDefaultTarget bool
@@ -280,24 +361,25 @@ func (restore *MongoRestore) Restore() error {
 		target, err = newActualPath(restore.TargetDirectory)
 		if err != nil {
 			if usedDefaultTarget {
-				log.Logv(log.Always, "see mongorestore --help for usage information")
+				log.Logv(log.Always, util.ShortUsage("mongorestore"))
 			}
-			return fmt.Errorf("mongorestore target '%v' invalid: %v", restore.TargetDirectory, err)
+			return Result{Err: fmt.Errorf("mongorestore target '%v' invalid: %v", restore.TargetDirectory, err)}
 		}
 		// handle cases where the user passes in a file instead of a directory
 		if !target.IsDir() {
 			log.Logv(log.DebugLow, "mongorestore target is a file, not a directory")
 			err = restore.handleBSONInsteadOfDirectory(restore.TargetDirectory)
 			if err != nil {
-				return err
+				return Result{Err: err}
 			}
 		} else {
 			log.Logv(log.DebugLow, "mongorestore target is a directory, not a file")
 		}
 	}
-	if restore.NSOptions.Collection != "" &&
+	if restore.ToolOptions.Namespace.Collection != "" &&
 		restore.OutputOptions.NumParallelCollections > 1 &&
-		restore.OutputOptions.NumInsertionWorkers == 1 {
+		restore.OutputOptions.NumInsertionWorkers == 1 &&
+		!restore.OutputOptions.MaintainInsertionOrder {
 		// handle special parallelization case when we are only restoring one collection
 		// by mapping -j to insertion workers rather than parallel collections
 		log.Logvf(log.DebugHigh,
@@ -308,7 +390,6 @@ func (restore *MongoRestore) Restore() error {
 	if restore.InputOptions.Archive != "" {
 		if int(restore.archive.Prelude.Header.ConcurrentCollections) > restore.OutputOptions.NumParallelCollections {
 			restore.OutputOptions.NumParallelCollections = int(restore.archive.Prelude.Header.ConcurrentCollections)
-			restore.OutputOptions.NumInsertionWorkers = int(restore.archive.Prelude.Header.ConcurrentCollections)
 			log.Logvf(log.Always,
 				"setting number of parallel collections to number of parallel collections in archive (%v)",
 				restore.archive.Prelude.Header.ConcurrentCollections,
@@ -319,32 +400,32 @@ func (restore *MongoRestore) Restore() error {
 	// Create the demux before intent creation, because muted archive intents need
 	// to register themselves with the demux directly
 	if restore.InputOptions.Archive != "" {
-		restore.archive.Demux = archive.CreateDemux(restore.archive.Prelude.NamespaceMetadatas, restore.archive.In)
+		restore.archive.Demux = archive.CreateDemux(restore.archive.Prelude.NamespaceMetadatas, restore.archive.In, restore.isAtlasProxy)
 	}
 
 	switch {
 	case restore.InputOptions.Archive != "":
 		log.Logvf(log.Always, "preparing collections to restore from")
 		err = restore.CreateAllIntents(target)
-	case restore.NSOptions.DB != "" && restore.NSOptions.Collection == "":
+	case restore.ToolOptions.Namespace.DB != "" && restore.ToolOptions.Namespace.Collection == "":
 		log.Logvf(log.Always,
 			"building a list of collections to restore from %v dir",
 			target.Path())
 		err = restore.CreateIntentsForDB(
-			restore.NSOptions.DB,
+			restore.ToolOptions.Namespace.DB,
 			target,
 		)
-	case restore.NSOptions.DB != "" && restore.NSOptions.Collection != "" && restore.TargetDirectory == "-":
+	case restore.ToolOptions.Namespace.DB != "" && restore.ToolOptions.Namespace.Collection != "" && restore.TargetDirectory == "-":
 		log.Logvf(log.Always, "setting up a collection to be read from standard input")
 		err = restore.CreateStdinIntentForCollection(
-			restore.NSOptions.DB,
-			restore.NSOptions.Collection,
+			restore.ToolOptions.Namespace.DB,
+			restore.ToolOptions.Namespace.Collection,
 		)
-	case restore.NSOptions.DB != "" && restore.NSOptions.Collection != "":
+	case restore.ToolOptions.Namespace.DB != "" && restore.ToolOptions.Namespace.Collection != "":
 		log.Logvf(log.Always, "checking for collection data in %v", target.Path())
 		err = restore.CreateIntentForCollection(
-			restore.NSOptions.DB,
-			restore.NSOptions.Collection,
+			restore.ToolOptions.Namespace.DB,
+			restore.ToolOptions.Namespace.Collection,
 			target,
 		)
 	default:
@@ -352,26 +433,26 @@ func (restore *MongoRestore) Restore() error {
 		err = restore.CreateAllIntents(target)
 	}
 	if err != nil {
-		return fmt.Errorf("error scanning filesystem: %v", err)
+		return Result{Err: fmt.Errorf("error scanning filesystem: %v", err)}
 	}
 
-	if restore.isMongos && restore.manager.HasConfigDBIntent() && restore.NSOptions.DB == "" {
-		return fmt.Errorf("cannot do a full restore on a sharded system - " +
-			"remove the 'config' directory from the dump directory first")
+	if restore.isMongos && restore.manager.HasConfigDBIntent() && restore.ToolOptions.Namespace.DB == "" {
+		return Result{Err: fmt.Errorf("cannot do a full restore on a sharded system - " +
+			"remove the 'config' directory from the dump directory first")}
 	}
 
 	if restore.InputOptions.OplogFile != "" {
 		err = restore.CreateIntentForOplog()
 		if err != nil {
-			return fmt.Errorf("error reading oplog file: %v", err)
+			return Result{Err: fmt.Errorf("error reading oplog file: %v", err)}
 		}
 	}
 	if restore.InputOptions.OplogReplay && restore.manager.Oplog() == nil {
-		return fmt.Errorf("no oplog file to replay; make sure you run mongodump with --oplog")
+		return Result{Err: fmt.Errorf("no oplog file to replay; make sure you run mongodump with --oplog")}
 	}
 	if restore.manager.GetOplogConflict() {
-		return fmt.Errorf("cannot provide both an oplog.bson file and an oplog file with --oplogFile, " +
-			"nor can you provide both a local/oplog.rs.bson and a local/oplog.$main.bson file.")
+		return Result{Err: fmt.Errorf("cannot provide both an oplog.bson file and an oplog file with --oplogFile, " +
+			"nor can you provide both a local/oplog.rs.bson and a local/oplog.$main.bson file")}
 	}
 
 	conflicts := restore.manager.GetDestinationConflicts()
@@ -379,12 +460,12 @@ func (restore *MongoRestore) Restore() error {
 		for _, conflict := range conflicts {
 			log.Logvf(log.Always, "%s", conflict.Error())
 		}
-		return fmt.Errorf("cannot restore with conflicting namespace destinations")
+		return Result{Err: fmt.Errorf("cannot restore with conflicting namespace destinations")}
 	}
 
 	if restore.OutputOptions.DryRun {
 		log.Logvf(log.Always, "dry run completed")
-		return nil
+		return Result{}
 	}
 
 	demuxFinished := make(chan interface{})
@@ -407,12 +488,15 @@ func (restore *MongoRestore) Restore() error {
 			ns, ok := <-namespaceChan
 			// the archive can have only special collections. In that case we keep reading until
 			// the namespaces are exhausted, indicated by the namespaceChan being closed.
+			log.Logvf(log.DebugLow, "received %v from namespaceChan", ns)
 			if !ok {
 				break
 			}
+			dbName, collName := util.SplitNamespace(ns)
+			ns = dbName + "." + strings.TrimPrefix(collName, "system.buckets.")
 			intent := restore.manager.IntentForNamespace(ns)
 			if intent == nil {
-				return fmt.Errorf("no intent for collection in archive: %v", ns)
+				return Result{Err: fmt.Errorf("no intent for collection in archive: %v", ns)}
 			}
 			if intent.IsSystemIndexes() ||
 				intent.IsUsers() ||
@@ -436,46 +520,64 @@ func (restore *MongoRestore) Restore() error {
 		log.Logv(log.Info, "comparing auth version of the dump directory and target server")
 		restore.authVersions.Dump, err = restore.GetDumpAuthVersion()
 		if err != nil {
-			return fmt.Errorf("error getting auth version from dump: %v", err)
+			return Result{Err: fmt.Errorf("error getting auth version from dump: %v", err)}
 		}
 		restore.authVersions.Server, err = auth.GetAuthVersion(restore.SessionProvider)
 		if err != nil {
-			return fmt.Errorf("error getting auth version of server: %v", err)
+			return Result{Err: fmt.Errorf("error getting auth version of server: %v", err)}
 		}
 		err = restore.ValidateAuthVersions()
 		if err != nil {
-			return fmt.Errorf(
+			return Result{Err: fmt.Errorf(
 				"the users and roles collections in the dump have an incompatible auth version with target server: %v",
-				err)
+				err)}
 		}
 	}
 
 	err = restore.LoadIndexesFromBSON()
 	if err != nil {
-		return fmt.Errorf("restore error: %v", err)
+		return Result{Err: fmt.Errorf("restore error: %v", err)}
+	}
+
+	err = restore.PopulateMetadataForIntents()
+	if err != nil {
+		return Result{Err: fmt.Errorf("restore error: %v", err)}
+	}
+
+	err = restore.preFlightChecks()
+	if err != nil {
+		return Result{Err: fmt.Errorf("restore error: %v", err)}
 	}
 
 	// Restore the regular collections
 	if restore.InputOptions.Archive != "" {
 		restore.manager.UsePrioritizer(restore.archive.Demux.NewPrioritizer(restore.manager))
 	} else if restore.OutputOptions.NumParallelCollections > 1 {
-		restore.manager.Finalize(intents.MultiDatabaseLTF)
+		// 3.0+ has collection-level locking for writes, so it is most efficient to
+		// prioritize by collection size. Pre-3.0 we try to avoid inserting into collections
+		// in the same database simultaneously due to the database-level locking.
+		// Up to 4.2, foreground index builds take a database-level lock for the entire build,
+		// but this prioritizer is not used for index builds so we don't need to worry about that here.
+		if restore.serverVersion.GTE(db.Version{3, 0, 0}) {
+			restore.manager.Finalize(intents.LongestTaskFirst)
+		} else {
+			restore.manager.Finalize(intents.MultiDatabaseLTF)
+		}
 	} else {
 		// use legacy restoration order if we are single-threaded
 		restore.manager.Finalize(intents.Legacy)
 	}
 
-	restore.termChan = make(chan struct{})
-
-	if err := restore.RestoreIntents(); err != nil {
-		return err
+	result := restore.RestoreIntents()
+	if result.Err != nil {
+		return result
 	}
 
 	// Restore users/roles
 	if restore.ShouldRestoreUsersAndRoles() {
 		err = restore.RestoreUsersOrRoles(restore.manager.Users(), restore.manager.Roles())
 		if err != nil {
-			return fmt.Errorf("restore error: %v", err)
+			return result.withErr(fmt.Errorf("restore error: %v", err))
 		}
 	}
 
@@ -483,15 +585,72 @@ func (restore *MongoRestore) Restore() error {
 	if restore.InputOptions.OplogReplay {
 		err = restore.RestoreOplog()
 		if err != nil {
-			return fmt.Errorf("restore error: %v", err)
+			return result.withErr(fmt.Errorf("restore error: %v", err))
 		}
 	}
 
-	defer log.Logv(log.Always, "done")
+	if !restore.OutputOptions.NoIndexRestore {
+		err = restore.RestoreIndexes()
+		if err != nil {
+			return result.withErr(err)
+		}
+	}
 
 	if restore.InputOptions.Archive != "" {
 		<-demuxFinished
-		return demuxErr
+		return result.withErr(demuxErr)
+	}
+
+	return result
+}
+
+func (restore *MongoRestore) preFlightChecks() error {
+
+	for _, intent := range restore.manager.Intents() {
+		if intent.Type == "timeseries" {
+
+			if !restore.OutputOptions.Drop {
+				timeseriesExists, err := restore.CollectionExists(intent.DB, intent.C)
+				if err != nil {
+					return err
+				}
+
+				if timeseriesExists {
+					return fmt.Errorf("timeseries collection `%s` already exists on the destination. "+
+						"You must remove this collection from the destination or use --drop", intent.Namespace())
+				}
+
+				bucketExists, err := restore.CollectionExists(intent.DB, intent.DataCollection())
+				if err != nil {
+					return err
+				}
+
+				if bucketExists {
+					return fmt.Errorf("system.buckets collection `%v` already exists on the destination. "+
+						"You must remove this collection from the destination in order to restore %s", intent.DataNamespace(), intent.Namespace())
+				}
+			}
+
+			if restore.OutputOptions.NoOptionsRestore {
+				return fmt.Errorf("cannot specify --noOptionsRestore when restoring timeseries collections")
+			}
+		}
+	}
+
+	if restore.serverVersion.GTE(db.Version{4, 9, 0}) && !restore.OutputOptions.NoIndexRestore {
+		namespaces := restore.indexCatalog.Namespaces()
+		for _, ns := range namespaces {
+			indexes := restore.indexCatalog.GetIndexes(ns.DB, ns.Collection)
+			for _, index := range indexes {
+				for _, keyElement := range index.Key {
+					if keyElement.Value == "geoHaystack" {
+						return fmt.Errorf("found a geoHaystack index: %v on %s. "+
+							"geoHaystack indexes are not supported by the destination cluster. "+
+							"Remove the index from the source or use --noIndexRestore to skip all indexes.", index.Key, ns.String())
+					}
+				}
+			}
+		}
 	}
 
 	return nil
@@ -499,7 +658,7 @@ func (restore *MongoRestore) Restore() error {
 
 func (restore *MongoRestore) getArchiveReader() (rc io.ReadCloser, err error) {
 	if restore.InputOptions.Archive == "-" {
-		rc = ioutil.NopCloser(restore.stdin)
+		rc = ioutil.NopCloser(restore.InputReader)
 	} else {
 		targetStat, err := os.Stat(restore.InputOptions.Archive)
 		if err != nil {
@@ -532,7 +691,5 @@ func (restore *MongoRestore) getArchiveReader() (rc io.ReadCloser, err error) {
 }
 
 func (restore *MongoRestore) HandleInterrupt() {
-	if restore.termChan != nil {
-		close(restore.termChan)
-	}
+	restore.terminate = true
 }
