@@ -34,7 +34,7 @@
 #   jobs on the same host at once.  So something's gotta change.
 
 from datetime import datetime
-from itertools import izip
+
 import glob
 import logging
 from optparse import OptionParser
@@ -55,11 +55,11 @@ from pymongo import MongoClient
 from pymongo.errors import OperationFailure
 from pymongo import ReadPreference
 
-import cleanbb
-import utils
+from . import cleanbb
+from . import utils
 
 try:
-    import cPickle as pickle
+    import pickle as pickle
 except ImportError:
     import pickle
 
@@ -133,17 +133,17 @@ class NullMongod(object):
 
 
 def dump_stacks(signal, frame):
-    print "======================================"
-    print "DUMPING STACKS due to SIGUSR1 signal"
-    print "======================================"
+    print("======================================")
+    print("DUMPING STACKS due to SIGUSR1 signal")
+    print("======================================")
     threads = threading.enumerate();
 
-    print "Total Threads: " + str(len(threads))
+    print("Total Threads: " + str(len(threads)))
 
-    for id, stack in sys._current_frames().items():
-        print "Thread %d" % (id)
-        print "".join(traceback.format_stack(stack))
-    print "======================================"
+    for id, stack in list(sys._current_frames().items()):
+        print("Thread %d" % (id))
+        print("".join(traceback.format_stack(stack)))
+    print("======================================")
 
 
 def buildlogger(cmd, is_global=False):
@@ -164,7 +164,7 @@ def clean_dbroot(dbroot="", nokill=False):
     if clean_whole_dbroot and not (small_oplog or small_oplog_rs):
         dbroot = os.path.normpath(smoke_db_prefix + "/data/db")
     if os.path.exists(dbroot):
-        print("clean_dbroot: %s" % dbroot)
+        print(("clean_dbroot: %s" % dbroot))
         cleanbb.cleanup(dbroot, nokill)
 
 
@@ -196,8 +196,8 @@ class mongod(NullMongod):
         try:
             self.check_mongo_port(int(port))
             return True
-        except Exception,e:
-            print >> sys.stderr, e
+        except Exception as e:
+            print(e, file=sys.stderr)
             return False
         
     def did_mongod_start(self, port=mongod_port, timeout=300):
@@ -207,14 +207,14 @@ class mongod(NullMongod):
             if is_up:
                 return True
             timeout = timeout - 1
-        print >> sys.stderr, "timeout starting mongod"
+        print("timeout starting mongod", file=sys.stderr)
         return False
 
     def start(self):
         global mongod_port
         global mongod
         if self.proc:
-            print >> sys.stderr, "probable bug: self.proc already set in start()"
+            print("probable bug: self.proc already set in start()", file=sys.stderr)
             return
         self.ensure_test_dirs()
         dir_name = smoke_db_prefix + "/data/db/sconsTests/"
@@ -253,8 +253,6 @@ class mongod(NullMongod):
             argv += ["--replSet", "foo", "--oplogSize", "511"]
         if self.kwargs.get('no_journal'):
             argv += ['--nojournal']
-        if self.kwargs.get('no_preallocj'):
-            argv += ['--nopreallocj']
         if self.kwargs.get('auth'):
             argv += ['--auth', '--setParameter', 'enableLocalhostAuthBypass=false']
             authMechanism = self.kwargs.get('authMechanism', 'SCRAM-SHA-1')
@@ -268,10 +266,15 @@ class mongod(NullMongod):
                      '--sslPEMKeyFile', 'jstests/libs/server.pem',
                      '--sslCAFile', 'jstests/libs/ca.pem',
                      '--sslWeakCertificateValidation']
+        if self.kwargs.get('use_tls'):
+             argv += ['--tlsMode', "requireTLS",
+                      '--tlsCertificateKeyFile', 'jstests/libs/server.pem',
+                      '--tlsCAFile', 'jstests/libs/ca.pem',
+                      '--tlsAllowConnectionsWithoutCertificates']
         if self.kwargs.get('use_x509'):
             argv += ['--clusterAuthMode','x509'];
             self.auth = True
-        print "running " + " ".join(argv)
+        print("running " + " ".join(argv))
         self.proc = self._start(buildlogger(argv, is_global=True))
 
         # If the mongod process is spawned under buildlogger.py, then the first line of output
@@ -353,7 +356,7 @@ class mongod(NullMongod):
 
     def stop(self):
         if not self.proc:
-            print >> sys.stderr, "probable bug: self.proc unset in stop()"
+            print("probable bug: self.proc unset in stop()", file=sys.stderr)
             return
         try:
             if os.sys.platform == "win32" and self.job_object is not None:
@@ -408,9 +411,9 @@ class mongod(NullMongod):
                 self.proc.terminate()
             else:
                 os.kill(self.proc.pid, 15)
-        except Exception, e:
-            print >> sys.stderr, "error shutting down mongod"
-            print >> sys.stderr, e
+        except Exception as e:
+            print("error shutting down mongod", file=sys.stderr)
+            print(e, file=sys.stderr)
         self.proc.wait()
 
         if self._stdout_pipe is not None:
@@ -425,12 +428,12 @@ class mongod(NullMongod):
         # anyway.
         retcode = self.proc.returncode
         if os.sys.platform != "win32" and retcode != 0:
-            raise(Exception('mongod process exited with non-zero code %d' % retcode))
+            raise Exception
 
     def wait_for_repl(self):
-        print "Awaiting replicated (w:2, wtimeout:5min) insert (port:" + str(self.port) + ")"
+        print("Awaiting replicated (w:2, wtimeout:5min) insert (port:" + str(self.port) + ")")
         MongoClient(port=self.port).testing.smokeWait.insert({}, w=2, wtimeout=5*60*1000)
-        print "Replicated write completed -- done wait_for_repl"
+        print("Replicated write completed -- done wait_for_repl")
 
 class Bug(Exception):
     def __str__(self):
@@ -458,7 +461,7 @@ class TestServerFailure(TestFailure):
 def check_db_hashes(master, slave):
     # Need to pause a bit so a slave might catch up...
     if not slave.slave:
-        raise(Bug("slave instance doesn't have slave attribute set"))
+        raise Bug
 
     master.wait_for_repl()
 
@@ -470,7 +473,7 @@ def check_db_hashes(master, slave):
 
     global lost_in_slave, lost_in_master, screwy_in_slave, replicated_collections
 
-    replicated_collections += master.dict.keys()
+    replicated_collections += list(master.dict.keys())
 
     for coll in replicated_collections:
         if coll not in slave.dict and coll not in lost_in_slave:
@@ -490,13 +493,13 @@ def check_db_hashes(master, slave):
                 sDocs = list(sTestDB[coll].find().sort("_id", 1))
                 mDiffDocs = list()
                 sDiffDocs = list()
-                for left, right in izip(mDocs, sDocs):
+                for left, right in zip(mDocs, sDocs):
                     if left != right:
                         mDiffDocs.append(left)
                         sDiffDocs.append(right)
 
                 stats["docs"] = {'master': mDiffDocs, 'slave': sDiffDocs }
-            except Exception, e:
+            except Exception as e:
                 stats["error-docs"] = e;
 
             screwy_in_slave[coll] = stats
@@ -504,12 +507,12 @@ def check_db_hashes(master, slave):
                 mOplog = mTestDB.connection.local["oplog.$main"];
                 oplog_entries = list(mOplog.find({"$or": [{"ns":mTestDB[coll].full_name}, \
                                                           {"op":"c"}]}).sort("$natural", 1))
-                print "oplog for %s" % mTestDB[coll].full_name
+                print("oplog for %s" % mTestDB[coll].full_name)
                 for doc in oplog_entries:
                     pprint.pprint(doc, width=200)
 
 
-    for db in slave.dict.keys():
+    for db in list(slave.dict.keys()):
         if db not in master.dict and db not in lost_in_master:
             lost_in_master.append(db)
 
@@ -528,7 +531,7 @@ def skipTest(path):
         if basename in ["cursor8.js", "indexh.js", "dropdb.js", "dropdb_race.js", 
                         "connections_opened.js", "opcounters_write_cmd.js", "dbadmin.js"]:
             return True
-    if use_ssl:
+    if use_ssl or use_tls:
         # Skip tests using mongobridge since it does not support SSL
         # TODO: Remove when SERVER-10910 has been resolved.  
         if basename in ["gridfs.js", "initial_sync3.js", "majority.js", "no_chaining.js",
@@ -617,6 +620,11 @@ def runTest(test, result):
                      "--sslPEMKeyFile", "jstests/libs/client.pem",
                      "--sslCAFile", "jstests/libs/ca.pem",
                      "--sslAllowInvalidCertificates"]
+        if use_tls:
+            argv += ["--tls",
+                     "--tlsCertificateKeyFile", "jstests/libs/client.pem",
+                     "--tlsCAFile", "jstests/libs/ca.pem",
+                     "--tlsAllowInvalidCertificates"]
         argv += [path]
     elif ext in ["", ".exe"]:
         # Blech.
@@ -656,8 +664,7 @@ def runTest(test, result):
     # FIXME: we don't handle the case where the subprocess
     # hangs... that's bad.
     if ( argv[0].endswith( 'mongo' ) or argv[0].endswith( 'mongo.exe' ) ) and not '--eval' in argv :
-        evalString = 'load("jstests/libs/servers.js");load("jstests/libs/servers_misc.js");' +\
-                     'TestData = new Object();' + \
+        evalString = 'TestData = new Object();' + \
                      'TestData.storageEngine = "' + ternary( storage_engine, storage_engine, "" ) + '";' + \
                      'TestData.wiredTigerEngineConfigString = "' + ternary( wiredtiger_engine_config_string, wiredtiger_engine_config_string, "" ) + '";' + \
                      'TestData.wiredTigerCollectionConfigString = "' + ternary( wiredtiger_collection_config_string, wiredtiger_collection_config_string, "" ) + '";' + \
@@ -675,6 +682,7 @@ def runTest(test, result):
                      'TestData.authMechanism = ' + ternary( authMechanism,
                                                '"' + str(authMechanism) + '"', 'null') + ";" + \
                      'TestData.useSSL = ' + ternary( use_ssl ) + ";" + \
+                     'TestData.useTLS = ' + ternary( use_tls ) + ";" + \
                      'TestData.useX509 = ' + ternary( use_x509 ) + ";"
         # this updates the default data directory for mongod processes started through shell (src/mongo/shell/servers.js)
         evalString += 'MongoRunner.dataDir = "' + os.path.abspath(smoke_db_prefix + '/data/db') + '";'
@@ -697,8 +705,6 @@ def runTest(test, result):
 
 
     if argv[0].endswith( 'dbtest' ) or argv[0].endswith( 'dbtest.exe' ):
-        if no_preallocj :
-            argv = argv + [ '--nopreallocj' ]
         if temp_path:
             argv = argv + [ '--tempPath', temp_path ]
 
@@ -753,7 +759,7 @@ def runTest(test, result):
 
     is_mongod_still_up = test_mongod.is_mongod_up(mongod_port)
     if start_mongod and not is_mongod_still_up:
-        print "mongod is not running after test"
+        print("mongod is not running after test")
         result["mongod_running_at_end"] = is_mongod_still_up;
         raise TestServerFailure(path)
 
@@ -762,7 +768,7 @@ def runTest(test, result):
     if r != 0:
         raise TestExitFailure(path, r)
 
-    print ""
+    print("")
 
 def run_tests(tests):
     # FIXME: some suites of tests start their own mongod, so don't
@@ -791,6 +797,7 @@ def run_tests(tests):
                             authMechanism=authMechanism,
                             keyFile=keyFile,
                             use_ssl=use_ssl,
+                            use_tls=use_tls,
                             use_x509=use_x509)
             master.start()
 
@@ -819,6 +826,7 @@ def run_tests(tests):
                            authMechanism=authMechanism,
                            keyFile=keyFile,
                            use_ssl=use_ssl,
+                           use_tls=use_tls,
                            use_x509=use_x509)
             slave.start()
             primary = MongoClient(port=master.port);
@@ -833,7 +841,7 @@ def run_tests(tests):
                 result = primary.admin.command("ismaster");
                 ismaster = result["ismaster"]
                 if not ismaster:
-                    print "waiting for primary to be available ..."
+                    print("waiting for primary to be available ...")
                     time.sleep(.2)
             
             secondaryUp = False
@@ -843,7 +851,7 @@ def run_tests(tests):
                 result = sConn.admin.command("ismaster");
                 secondaryUp = result["secondary"]
                 if not secondaryUp:
-                    print "waiting for secondary to be available ..."
+                    print("waiting for secondary to be available ...")
                     time.sleep(.2)
 
         if small_oplog or small_oplog_rs:
@@ -865,7 +873,7 @@ def run_tests(tests):
                 if skipTest(test_path):
                     test_result["status"] = "skip"
 
-                    print "skipping " + test_path
+                    print("skipping " + test_path)
                 else:
                     fails.append(test)
                     runTest(test, test_result)
@@ -902,23 +910,24 @@ def run_tests(tests):
                                         authMechanism=authMechanism,
                                         keyFile=keyFile,
                                         use_ssl=use_ssl,
+                                        use_tls=use_tls,
                                         use_x509=use_x509)
                         master.start()
 
-            except TestFailure, f:
+            except TestFailure as f:
                 test_result["end"] = time.time()
                 test_result["elapsed"] = test_result["end"] - test_result["start"]
                 test_result["error"] = str(f)
                 test_result["status"] = "fail"
                 test_report["results"].append( test_result )
                 try:
-                    print f
+                    print(f)
                     # Record the failing test and re-raise.
                     losers[f.path] = f.status
                     raise f
-                except TestServerFailure, f:
+                except TestServerFailure as f:
                     return 2
-                except TestFailure, f:
+                except TestFailure as f:
                     if not continue_on_failure:
                         return 1
         if isinstance(slave, mongod):
@@ -933,52 +942,52 @@ def run_tests(tests):
 def check_and_report_replication_dbhashes():
     def missing(lst, src, dst):
         if lst:
-            print """The following collections were present in the %s but not the %s
-at the end of testing:""" % (src, dst)
+            print("""The following collections were present in the %s but not the %s
+at the end of testing:""" % (src, dst))
             for db in lst:
-                print db
+                print(db)
 
     missing(lost_in_slave, "master", "slave")
     missing(lost_in_master, "slave", "master")
     if screwy_in_slave:
-        print """The following collections has different hashes in master and slave
-at the end of testing:"""
-        for coll in screwy_in_slave.keys():
+        print("""The following collections has different hashes in master and slave
+at the end of testing:""")
+        for coll in list(screwy_in_slave.keys()):
             stats = screwy_in_slave[coll]
             # Counts are "approx" because they are collected after the dbhash runs and may not
             # reflect the states of the collections that were hashed. If the hashes differ, one
             # possibility is that a test exited with writes still in-flight.
-            print "collection: %s\t (master/slave) hashes: %s/%s counts (approx): %i/%i" % (coll, stats['hashes']['master'], stats['hashes']['slave'], stats['counts']['master'], stats['counts']['slave'])
+            print("collection: %s\t (master/slave) hashes: %s/%s counts (approx): %i/%i" % (coll, stats['hashes']['master'], stats['hashes']['slave'], stats['counts']['master'], stats['counts']['slave']))
             if "docs" in stats:
                 if (("master" in stats["docs"] and len(stats["docs"]["master"]) != 0) or
                     ("slave" in stats["docs"] and len(stats["docs"]["slave"]) != 0)):
-                    print "All docs matched!"
+                    print("All docs matched!")
                 else:
-                    print "Different Docs"
-                    print "Master docs:"
+                    print("Different Docs")
+                    print("Master docs:")
                     pprint.pprint(stats["docs"]["master"], indent=2)
-                    print "Slave docs:"
+                    print("Slave docs:")
                     pprint.pprint(stats["docs"]["slave"], indent=2)
             if "error-docs" in stats:
-                print "Error getting docs to diff:"
+                print("Error getting docs to diff:")
                 pprint.pprint(stats["error-docs"])
         return True
 
     if (small_oplog or small_oplog_rs) and not (lost_in_master or lost_in_slave or screwy_in_slave):
-        print "replication ok for %d collections" % (len(replicated_collections))
+        print("replication ok for %d collections" % (len(replicated_collections)))
 
     return False
 
 
 def report():
-    print "%d tests succeeded" % len(winners)
-    num_missed = len(tests) - (len(winners) + len(losers.keys()))
+    print("%d tests succeeded" % len(winners))
+    num_missed = len(tests) - (len(winners) + len(list(losers.keys())))
     if num_missed:
-        print "%d tests didn't get run" % num_missed
+        print("%d tests didn't get run" % num_missed)
     if losers:
-        print "The following tests failed (with exit code):"
+        print("The following tests failed (with exit code):")
         for loser in losers:
-            print "%s\t%d" % (loser, losers[loser])
+            print("%s\t%d" % (loser, losers[loser]))
 
     test_result = { "start": time.time() }
     if check_and_report_replication_dbhashes():
@@ -1010,6 +1019,7 @@ suiteGlobalConfig = {   "files": ("files/*.js", False),
                         "oplog": ("oplog/*.js", False),
                         "import": ("import/*.js", False),
                         "ssl": ("ssl/*.js", False),
+                        "tls": ("tls/*.js", False),
                         "unstable": ("unstable/*.js", False),
                     }
 
@@ -1146,7 +1156,7 @@ def set_globals(options, tests):
     global small_oplog, small_oplog_rs
     global no_journal, set_parameters, set_parameters_mongos, no_preallocj, storage_engine, wiredtiger_engine_config_string, wiredtiger_collection_config_string, wiredtiger_index_config_string
     global auth, authMechanism, keyFile, keyFileData, smoke_db_prefix, test_path, start_mongod
-    global use_ssl, use_x509
+    global use_ssl, use_tls, use_x509
     global file_of_commands_mode
     global report_file, shell_write_mode, use_write_commands
     global temp_path
@@ -1156,6 +1166,8 @@ def set_globals(options, tests):
     start_mongod = options.start_mongod
     if hasattr(options, 'use_ssl'):
         use_ssl = options.use_ssl
+    if hasattr(options, 'use_tls'):
+        use_tls = options.use_tls
     if hasattr(options, 'use_x509'):
         use_x509 = options.use_x509
         use_ssl = use_ssl or use_x509
@@ -1238,7 +1250,7 @@ def run_old_fails():
         return # This counts as passing so we will run all tests
 
     if ('version' not in state or state['version'] != file_version()):
-        print "warning: old version of failfile.smoke detected. skipping recent fails"
+        print("warning: old version of failfile.smoke detected. skipping recent fails")
         clear_failfile()
         return
 
@@ -1302,7 +1314,7 @@ def main():
     try:
         signal.signal(signal.SIGUSR1, dump_stacks)
     except AttributeError:
-        print "Cannot catch signals on Windows"
+        print("Cannot catch signals on Windows")
 
     parser = OptionParser(usage="usage: smoke.py [OPTIONS] ARGS*")
     parser.add_option('--mode', dest='mode', default='suite',
@@ -1343,9 +1355,6 @@ def main():
     parser.add_option('--nojournal', dest='no_journal', default=False,
                       action="store_true",
                       help='Do not turn on journaling in tests')
-    parser.add_option('--nopreallocj', dest='no_preallocj', default=False,
-                      action="store_true",
-                      help='Do not preallocate journal files in tests')
     parser.add_option('--auth', dest='auth', default=False,
                       action="store_true",
                       help='Run standalone mongods in tests with authentication enabled')
@@ -1376,6 +1385,9 @@ def main():
     parser.add_option('--use-ssl', dest='use_ssl', default=False,
                       action='store_true',
                       help='Run mongo shell and mongod instances with SSL encryption')
+    parser.add_option('--use-tls', dest='use_tls', default=False,
+                      action='store_true',
+                      help='Run mongo shell and mongod instances with TLS encryption')
     parser.add_option('--set-parameters', dest='set_parameters', default="",
                       help='Adds --setParameter to mongod for each passed in item in the csv list - ex. "param1=1,param2=foo" ')
     parser.add_option('--set-parameters-mongos', dest='set_parameters_mongos', default="",
@@ -1445,19 +1457,19 @@ def main():
 
     if options.ignore_files != None :
         ignore_patt = re.compile( options.ignore_files )
-        print "Ignoring files with pattern: ", ignore_patt
+        print("Ignoring files with pattern: ", ignore_patt)
 
         def ignore_test( test ):
             if ignore_patt.search( test[0] ) != None:
-                print "Ignoring test ", test[0]
+                print("Ignoring test ", test[0])
                 return False
             else:
                 return True
 
-        tests = filter( ignore_test, tests )
+        tests = list(filter( ignore_test, tests ))
 
     if not tests:
-        print "warning: no tests specified"
+        print("warning: no tests specified")
         return
 
     if options.with_cleanbb:
@@ -1472,7 +1484,7 @@ def main():
 
         test_report["end"] = time.time()
         test_report["elapsed"] = test_report["end"] - test_report["start"]
-        test_report["failures"] = len(losers.keys())
+        test_report["failures"] = len(list(losers.keys()))
         test_report["mongod_running_at_end"] = mongod().is_mongod_up(mongod_port)
         if report_file:
             f = open( report_file, "wb" )
