@@ -5,7 +5,16 @@
 
   jsTest.log('Testing exporting to stdout');
 
-  var toolTest = new ToolTest('stdout');
+  var TOOLS_TEST_CONFIG = {};
+  if (TestData.useTLS) {
+    TOOLS_TEST_CONFIG = {
+      tlsMode: "requireTLS",
+      tlsCertificateKeyFile: "jstests/libs/client.pem",
+      tlsCAFile: "jstests/libs/ca.pem",
+      tlsAllowInvalidHostnames: "",
+    };
+  }
+  var toolTest = new ToolTest('stdout', TOOLS_TEST_CONFIG);
   toolTest.startDB('foo');
 
   // the db and collection we'll use
@@ -22,12 +31,17 @@
   assert.eq(20, testColl.count());
 
   // export the data, writing to stdout
-  var ret = toolTest.runTool('export', '--db', 'test', '--collection', 'data');
+  var ret = toolTest.runTool('export', '--db', 'test',
+      '--collection', 'data',
+      '--ssl',
+      '--sslPEMKeyFile=jstests/libs/client.pem',
+      '--sslCAFile=jstests/libs/ca.pem',
+      '--sslAllowInvalidHostnames');
   assert.eq(0, ret);
 
   // wait for full output to appear
   assert.strContains.soon('exported 20 records', rawMongoProgramOutput,
-      'should show number of exported records');
+    'should show number of exported records');
 
   // grab the raw output
   var output = rawMongoProgramOutput();
